@@ -1,4 +1,4 @@
-/* Copyright (c) 2019-2023 Advanced Micro Devices, Inc.
+/* Copyright (c) 2019-2024 Advanced Micro Devices, Inc.
 
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
@@ -186,18 +186,25 @@ public:
   {
   private:
     const compute_queue_t &m_queue;
+    uint32_t m_xcc_id;
 
   public:
-    cwsr_record_t (const compute_queue_t &queue) : m_queue (queue) {}
+    cwsr_record_t (const compute_queue_t &queue, uint32_t xcc_id)
+      : m_queue (queue), m_xcc_id (xcc_id)
+    {
+    }
     /* cwsr_record_t is a polymorphic base class.  */
     virtual ~cwsr_record_t () = default;
 
+    /* Indicates if content of TTMP registers which should be initialized by
+       SPI are meaningful or cleared by dbgapi and should be ignored.  */
+    virtual bool spi_ttmps_setup_enabled () const = 0;
     /* Return the globally unique wave identifier.  */
     virtual amd_dbgapi_wave_id_t id () const = 0;
     /* The 3-dimensional workgroup coordinates.  */
-    virtual std::array<uint32_t, 3> group_ids () const = 0;
-    /* Return the record's postion in the workgroup.  */
-    virtual uint32_t position_in_group () const = 0;
+    virtual std::optional<std::array<uint32_t, 3>> group_ids () const = 0;
+    /* Return the record's position in the workgroup.  */
+    virtual std::optional<uint32_t> position_in_group () const = 0;
 
     /* Number of work-items in one wave.  */
     virtual size_t lane_count () const = 0;
@@ -212,7 +219,7 @@ public:
     virtual std::optional<amd_dbgapi_global_address_t>
     register_address (amdgpu_regnum_t regnum) const = 0;
 
-    /* Return true is a scratch slot is allocated for this record.  */
+    /* Return true if a scratch slot is allocated for this record.  */
     virtual bool is_scratch_enabled () const = 0;
 
     /* The shader engine ID this wave was created on.  */
@@ -225,6 +232,8 @@ public:
     /* The address of the byte following the last byte in the context save. */
     virtual amd_dbgapi_global_address_t end () const = 0;
 
+    uint32_t xcc_id () const { return m_xcc_id; }
+
     const compute_queue_t &queue () const { return m_queue; }
     const agent_t &agent () const;
     process_t &process () const;
@@ -247,22 +256,25 @@ public:
   }
 
   virtual size_t control_stack_iterate (
-    compute_queue_t &queue, const uint32_t *control_stack,
+    compute_queue_t &queue, uint32_t xcc_id, const uint32_t *control_stack,
     size_t control_stack_words, amd_dbgapi_global_address_t wave_area_address,
     amd_dbgapi_size_t wave_area_size,
     const std::function<void (std::unique_ptr<const cwsr_record_t>)>
       &wave_callback) const = 0;
 
-  virtual amd_dbgapi_global_address_t dispatch_packet_address (
+  virtual std::optional<amd_dbgapi_global_address_t> dispatch_packet_address (
     const architecture_t::cwsr_record_t &cwsr_record) const = 0;
 
+  /* Default initialize the trap temporary registers normally set up by SPI.
+   */
+  virtual void initialize_spi_ttmps (const wave_t &wave) const = 0;
+
+  virtual void record_spi_ttmps_setup (const wave_t &wave, bool enabled) const = 0;
+
   /* Return true if the trap temporary registers used by the trap handler to
      communicate with the debugger API have been initialized.  */
   virtual bool
   are_trap_handler_ttmps_initialized (const wave_t &wave) const = 0;
-  /* Default initialize the trap temporary registers normally set up by SPI.
-   */
-  virtual void initialize_spi_ttmps (const wave_t &wave) const = 0;
   /* Default initialize the trap temporary registers normally set up by the
      trap handler.  */
   virtual void initialize_trap_handler_ttmps (const wave_t &wave) const = 0;
@@ -275,10 +287,11 @@ public:
 
   virtual std::pair<amd_dbgapi_size_t /* offset  */,
                     amd_dbgapi_size_t /* size  */>
-  scratch_memory_region (uint32_t compute_tmpring_size_register,
-                         uint32_t shader_engine_count,
-                         uint32_t shader_engine_id,
-                         uint32_t scoreboard_id) const = 0;
+  scratch_memory_region (const agent_t &,
+                         uint32_t compute_tmpring_size_register,
+                         uint32_t xcc_id, uint32_t shader_engine_id,
+                         uint32_t scoreboard_id) const
+    = 0;
 
   virtual bool
   is_address_space_supported (const address_space_t &address_space) const = 0;
