@@ -109,7 +109,6 @@ private:
   std::optional<amd_dbgapi_os_queue_packet_id_t> m_read_packet_id{};
   std::optional<amd_dbgapi_os_queue_packet_id_t> m_write_packet_id{};
   amd_dbgapi_global_address_t m_scratch_backing_memory_address{ 0 };
-  amd_dbgapi_size_t m_per_xcc_scratch_backing_memory_size{ 0 };
   uint32_t m_compute_tmpring_size{ 0 };
 
   /* The memory reserved by the thunk library for the debugger is used to store
@@ -545,12 +544,6 @@ aql_queue_t::queue_state_changed ()
           - offsetof (amd_queue_t, read_dispatch_id),
         &m_scratch_backing_memory_address);
 
-      process ().read_global_memory (
-        m_os_queue_info.read_pointer_address
-          + offsetof (amd_queue_t, scratch_backing_memory_byte_size)
-          - offsetof (amd_queue_t, read_dispatch_id),
-        &m_per_xcc_scratch_backing_memory_size);
-
       process ().read_global_memory (
         m_os_queue_info.read_pointer_address
           + offsetof (amd_queue_t, compute_tmpring_size)
@@ -804,7 +797,8 @@ aql_queue_t::update_waves ()
       if (control_stack_begin != control_stack_end)
         {
           log_info ("decoding %s's context save area #%u: "
-                    "ctrl_stk:[0x%llx..0x%llx[, wave_area:[0x%llx..0x%llx[",
+                    "ctrl_stk:[%#" PRIx64 "..%#" PRIx64 "[, "
+                    "wave_area:[%#" PRIx64 "..%#" PRIx64 "[",
                     to_cstring (id ()), xcc_id, control_stack_begin,
                     control_stack_end, wave_area_begin, wave_area_end);
 
@@ -869,18 +863,10 @@ std::pair<amd_dbgapi_global_address_t /* address */,
 aql_queue_t::scratch_memory_region (uint32_t xcc_id, uint32_t shader_engine_id,
                                     uint32_t scoreboard_id) const
 {
-  /* The scratch memory for this queue is evenly divided between all XCCs, so
-     each XCC has its own scratch base.  */
-  amd_dbgapi_global_address_t xcc_scratch_base
-    = m_scratch_backing_memory_address
-      + m_per_xcc_scratch_backing_memory_size * xcc_id;
-
   auto [offset, size] = architecture ().scratch_memory_region (
-    m_compute_tmpring_size,
-    agent ().os_info ().shader_engine_count / agent ().os_info ().xcc_count,
-    shader_engine_id, scoreboard_id);
+    agent (), m_compute_tmpring_size, xcc_id, shader_engine_id, scoreboard_id);
 
-  return { xcc_scratch_base + offset, size };
+  return { m_scratch_backing_memory_address + offset, size };
 }
 
 void
@@ -977,7 +963,7 @@ public:
 amd_dbgapi_os_queue_type_t
 unsupported_queue_t::type () const
 {
-  switch (os_queue_type (m_os_queue_info))
+  switch (m_os_queue_info.queue_type)
     {
     case os_queue_type_t::compute:
       return AMD_DBGAPI_OS_QUEUE_TYPE_AMD_PM4;
@@ -1042,7 +1028,7 @@ queue_t::create (std::optional<amd_dbgapi_queue_id_t> queue_id,
                  const agent_t &agent,
                  const os_queue_snapshot_entry_t &os_queue_info)
 {
-  switch (os_queue_type (os_queue_info))
+  switch (os_queue_info.queue_type)
     {
     case os_queue_type_t::compute_aql:
       return agent.process ().create<detail::aql_queue_t> (queue_id, agent,
@@ -1054,13 +1040,13 @@ queue_t::create (std::optional<amd_dbgapi_queue_id_t> queue_id,
     }
 }
 
-os_queue_id_t
+std::optional<os_queue_id_t>
 queue_t::os_queue_id () const
 {
   if (is_valid ())
     return m_os_queue_info.queue_id;
   else
-    return m_os_queue_info.queue_id | os_queue_invalid_mask;
+    return {};
 }
 
 void
@@ -1106,6 +1092,12 @@ void
 queue_t::get_info (amd_dbgapi_queue_info_t query, size_t value_size,
                    void *value) const
 {
+  /* The caller of this method (amd_dbgapi_queue_get_info) must ensure
+     that the queue is valid.  Since the queue is retreived using
+     amd::dbgapi::handle_object_set_t<amd::dbgapi::queue_t>::find, any invalid
+     queue would have been filtered at this stage.  */
+  dbgapi_assert (is_valid ());
+
   switch (query)
     {
     case AMD_DBGAPI_QUEUE_INFO_AGENT:
@@ -1125,9 +1117,14 @@ queue_t::get_info (amd_dbgapi_queue_info_t query, size_t value_size,
       return;
 
     case AMD_DBGAPI_QUEUE_INFO_OS_ID:
-      utils::get_info (value_size, value,
-                       static_cast<amd_dbgapi_os_queue_id_t> (os_queue_id ()));
-      return;
+      {
+        std::optional<amd_dbgapi_os_queue_id_t> os_id = os_queue_id ();
+        dbgapi_assert (os_id.has_value ());
+        utils::get_info (
+          value_size, value,
+          static_cast<amd_dbgapi_os_queue_id_t> (os_id.value ()));
+        return;
+      }
 
     case AMD_DBGAPI_QUEUE_INFO_ADDRESS:
       utils::get_info (value_size, value, address ());
